
import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:pc_plugins/controllers/simple_fb_gallery/types.dart';

import 'config.dart';

class SimpleDcFbController<T> extends StateNotifier<AsyncValue<List<T>>>{
  final Reader reader;
  final SimpleDcFbConfig<T> config;
  final String? myid;
  static const idKey="id";
  CollectionReference<Map<String,dynamic>> get fbCol=>config.fstore.collection(config.fbPath);
  DocumentReference fbDoc(String id)=>config.fstore.doc(config.fbPath+"/"+id);
  String getId(T t)=>config.toDocWithId(t)[idKey];

  SimpleDcFbController(this.reader,this.myid,this.config,{bool initialLoad=false}) : super(const AsyncValue.loading()){
    if(initialLoad)load();
  }

  Future<int?> load({sortChoice? sortBy,
    List<FilterFunc>filters=const [], bool loadMore=false})async{
    if(myid==null)return null;
    sortBy=sortBy??config.initialSortBy;
    loadMore=loadMore&&state.value!=null;

    var qr=fbCol.orderBy(sortBy?.key??FieldPath.documentId,
        descending: sortBy?.descend??false).limit(config.pageSize);
    filters.forEach((filter) {qr=filter(qr); });
    if(config.constFilters!=null)config.constFilters!(myid!).forEach((filter) {qr=filter(qr); });

    if(loadMore&&state.value!.isNotEmpty) {
      qr=qr.startAfter([config.toDocWithId(state.value!.last)[sortBy?.key??idKey]]);
    }
    QuerySnapshot<Map<String,dynamic>> r;
    List<T> added;
    if(config.onLoad!=null)config.onLoad!(true);
    try{
      r=await qr.get();
      ///handel FutureOr for FromDoc
      added=await Future.wait<T>(r.docs.map((e) =>
          Future(()=>config.fromDoc(e))));
      if(config.onLoad!=null)config.onLoad!(false);
    }on FirebaseException catch(e){
      config.onException(e);
      if(config.onLoad!=null)config.onLoad!(false);
      return null;
    }

    state=AsyncValue.data(loadMore?List.of(addMore(state.value,added)):added);
    if(r.size<config.pageSize&&config.onReachEnd!=null) {
      config.onReachEnd!(state.value!.length);
    }
    return added.length;
  }

  Future<T?> add(T t,{bool addToState=true})async{
    DocumentReference doc;
    try{
      doc=await fbCol.add(config.toDocWithId(t)..remove(idKey));
    }on FirebaseException catch(e){
      config.onException(e);
      return null;
    }
    final r = config.setId(t,doc.id);
    if(addToState) {
      state=state.whenData((v) => List.of(v)..insert(0, r));
    }
    return r;
  }

  Future<T?> edit(T t)async{
    final jsn=config.toDocWithId(t);
    try{
      await fbDoc(jsn[idKey]).set(jsn..remove(idKey));
    }on FirebaseException catch(e){
      config.onException(e);
      return null;
    }
    final i=(state.value??[]).indexWhere((e) => getId(e)==getId(t));
    if(i!=-1)
      state=state.whenData((v) => List.of(v..[i]=t));
    return t;
  }

  Future<T?> delete(T t)async{
    final jsn=config.toDocWithId(t);
    try{
      await fbDoc(jsn[idKey]).delete();
    }on FirebaseException catch(e){
      config.onException(e);
      return null;
    }
    final i=(state.value??[]).indexWhere((e) => getId(e)==getId(t));
    if(i!=-1)
      state=state.whenData((v) => List.of(v..removeAt(i)));
    return t;
  }

  List<T> addMore(List<T>? origin,List<T>add){
    origin=origin??[];
    origin.removeWhere((e) => add.any((ae) => getId(ae)==getId(e)));
    return origin..addAll(add);
  }
}
