


import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class PopupLayout extends ModalRoute {
  PopupLayout(
      {Key? key,
      this.bgColor,
      required this.child,
      this.transition = true,
      this.opacity = 0.6,
      this.top,
      this.bottom,
      this.left,
      this.right,
      this.height,
      this.width});

  bool transition;
  double? top;
  double? bottom;
  double? left;
  double? right;
  double? height;
  double? width;
  double opacity;
  Color? bgColor;
  final Widget child;

  @override
  Duration get transitionDuration => Duration(milliseconds: 27);
  @override
  bool get opaque => false;
  @override
  bool get barrierDismissible => false;
  @override
  Color? get barrierColor =>
      bgColor == null ? Colors.black.withOpacity(opacity) : bgColor;
  @override
  String? get barrierLabel => null;
  @override
  bool get maintainState => false;

  @override
  Widget buildPage(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
  ) {
    var size=MediaQuery.of(context).size;
    if(height!=null){
      if(top==null){
        if(bottom==null){
          this.top=(size.height-height!)/2;
          bottom=top;
        }else{
          top=size.height-height!-bottom!;
        }
      }else if(bottom==null)bottom=size.height-height!-top!;
    }else{
      top=(top??10);bottom=(bottom??20);
    }
    if(width!=null){
      if(left==null){
        if(right==null){
          this.left=(size.width-width!)/2;
          right=left;
        }else{
          left=size.width-width!-right!;
        }
      }else if(right==null)right=size.width-width!-left!;
    }else{
      left=(left??20);right=(right??20);
    }

    return GestureDetector(
      onTap: () {
        // call this method here to hide soft keyboard
        SystemChannels.textInput.invokeMethod('TextInput.hide');
      },
      child: Material(
        // This makes sure that text and other content follows the material style
        type: MaterialType.transparency,
        //type: MaterialType.canvas,
        // make sure that the overlay content is not cut off
        child: SafeArea(
          bottom: true,
          child: _buildOverlayContent(context),
        ),
      ),
    );
  }

  //the dynamic content pass by parameter
  Widget _buildOverlayContent(BuildContext context) {
    return Stack(children: [
      GestureDetector(
        onTap: Navigator.of(context).pop,
      ),
      GestureDetector(
          child: Container(
        margin: EdgeInsets.only(
            bottom: this.bottom!,
            left: this.left!,
            right: this.right!,
            top: this.top!),
        child: child,
      ))
    ]);
  }

  @override
  Widget buildTransitions(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation, Widget child) {
    // You can add your own animations for the overlay content
    return FadeTransition(
        opacity: animation,
        child: child,);
    return transition
        ? FadeTransition(
            opacity: animation,
            child: ScaleTransition(
              scale: animation,
              child: child,
            ))
        : SlideTransition(
            position: Tween<Offset>(
              begin: const Offset(1.0, 0.0),
              end: Offset.zero,
            ).animate(animation),
            child: SlideTransition(
              position: Tween<Offset>(
                begin: Offset.zero,
                end: const Offset(1.0, 0.0),
              ).animate(secondaryAnimation),
              child: child,
            ),
          );
  }
}

class PopupContent extends StatefulWidget {
  final Widget? content;
  PopupContent({
    Key? key,
    this.content,
  }) : super(key: key);
  _PopupContentState createState() => _PopupContentState();
}

class _PopupContentState extends State<PopupContent> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: widget.content,
    );
  }

//  showPopup(BuildContext context, Widget widget, String title,
//      {BuildContext popupContext}) {
//    Navigator.push(
//      context,
//      PopupLayout(
//        top: 30,
//        left: 30,
//        right: 30,
//        bottom: 50,
//        child: PopupContent(
//          content: Scaffold(
//            appBar: AppBar(
//              title: Text(title),
//              leading: new Builder(builder: (context) {
//                return IconButton(
//                  icon: Icon(Icons.arrow_back),
//                  onPressed: () {
//                    try {
//                      Navigator.pop(context); //close the popup
//                    } catch (e) {}
//                  },
//                );
//              }),
//              brightness: Brightness.light,
//            ),
//            resizeToAvoidBottomPadding: false,
//            body: widget,
//          ),
//        ),
//      ),
//    );
//  }
}


